/*
** Module   : DatetimeUserType.java
** Abstract : Custom user type to wrap P2J datetime for Hibernate access
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 GES 20130121 Created initial version. Maps P2J datetime to SQL timestamp.
*/

package com.goldencode.p2j.persist.type;

import java.sql.*;
import com.goldencode.p2j.util.*;

/**
 * Implementation of a custom, Hibernate user type for the {@link
 * com.goldencode.p2j.util.datetime P2J datetime}. This permits Hibernate to
 * convert this data type to and from the SQL TIMESTAMP data type. The unknown
 * value is mapped to SQL NULL.
 */
public final class DatetimeUserType
extends AbstractWrapperUserType
{
   /** Single element array containing the SQL type constant being mapped */
   private static final int[] SQL_TYPES = { Types.TIMESTAMP };
   
   /**
    * Get the array of SQL data type constants (in this case only a single
    * constant), which this classes uses to map a datetime to/from a database
    * record.
    *
    * @return  Array containing a single type constant:  TIMESTAMP.
    */
   public int[] sqlTypes()
   {
      return SQL_TYPES;
   }
   
   /**
    * Gets the Java data type class mapped by this custom type.
    *
    * @return  The {@link com.goldencode.p2j.util.datetime P2J datetime} class.
    */
   public Class returnedClass()
   {
      return datetime.class;
   }
   
   /**
    * Create and return a deep copy of the given <code>datetime</code> instance.
    *
    * @param   value
    *          The <code>datetime</code> instance to be copied.
    *
    * @return  A new instance of <code>datetime</code> which contains the
    *          same internal value as <code>value</code>.
    */
   public Object deepCopy(Object value)
   {
      return new datetime((datetime) value);
   }
   
   /**
    * Extract a <code>datetime</code> value from a result set at a particular
    * column.
    *
    * @param   rs
    *          Result set containing data to be extracted.
    * @param   column
    *          Name of the column at which data will be extracted.
    *
    * @return  Timestamp data extracted from the result set; may be
    *          <code>null</code>.
    *
    * @throws  SQLException
    *          if a database access error occurs.
    */
   protected Object getBaseValue(ResultSet rs, String column)
   throws SQLException
   {
      return rs.getTimestamp(column);
   }
   
   /**
    * Construct and return an instance of <code>datetime</code> which is
    * initialized to the unknown value.
    *
    * @return  A <code>datetime</code> instance which represents the unknown
    *          value.
    */
   protected BaseDataType instantiateUnknown()
   {
      return datetime.instantiateUnknownDatetime();
   }
   
   /**
    * Construct and return an instance of <code>datetime</code> which wraps
    * <code>value</code>.
    *
    * @param   value
    *          Timestamp value to be wrapped by a <code>datetime</code>;
    *          will not be <code>null</code>.
    *
    * @return  <code>value</code> wrapped in a <code>datetime</code>.
    */
   protected BaseDataType instantiateRegular(Object value)
   {
      // Timestamp is a subclass of java.util.Date and we don't ever care
      // about the nanoseconds part, so this should work
      return new datetime((java.util.Date) value);
   }
   
   /**
    * Set a <code>Timestamp</code> value into the given prepared statement,
    * based upon the <code>datetime</code> instance <code>value</code>.
    *
    * @param   ps
    *          Prepared statement into which we will set our data.
    * @param   value
    *          An <code>datetime</code> instance.
    * @param   index
    *          Index in the statement at which to write the data.
    *
    * @throws  SQLException
    *          if an error occurs setting data into the statement.
    */
   protected void bindNonNull(PreparedStatement ps,
                              BaseDataType      value,
                              int               index)
   throws SQLException
   {
      // Need a java.sql.Timestamp; datetime class deals in java.util.Date. The
      // Date is constructed using the same timezone used for the datetime to
      // ensure there is no drift to the previous date, due to timezone or
      // daylight savings offsets.
      datetime dt = (datetime) value;
      Timestamp sqlTimestamp = new Timestamp(dt.dateValue().getTime());
      
      ps.setTimestamp(index, sqlTimestamp);
   }
}
